# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Provide the correct reactor and ui integration."""

import sys

# Module used to include the resources into this file
# Unused import images_rc, pylint: disable=W0611
from ubuntuone.controlpanel.gui.qt.ui import images_rc
from ubuntuone.controlpanel.utils import (
    autoupdate,
    AUTOUPDATE_INTERVAL,
)
# pylint: enable=W0611
from ubuntuone.controlpanel.gui.qt.uniqueapp import UniqueApplication

# Invalid name "source", pylint: disable=C0103
if sys.platform == 'win32':
    from ubuntuone.controlpanel.gui.qt.main import windows
    source = windows
else:
    from ubuntuone.controlpanel.gui.qt.main import linux
    source = linux
# pylint: enable=C0103


def main(switch_to='', alert=False, minimized=False, with_icon=False):
    """Start the Qt reactor and open the main window."""
    # The following cannot be imported outside this function
    # because u1trial already provides a reactor.

    # The main loop MUST be initialized before importing the reactor
    app = UniqueApplication(sys.argv, "ubuntuone-control-panel")
    source.main(app)

    # Reimport 'qt4reactor', 'reactor', 'start', pylint: disable=W0404, F0401
    import qt4reactor
    qt4reactor.install()

    from twisted.internet import reactor
    from ubuntuone.controlpanel.gui.qt.gui import start
    # pylint: enable=W0404, F0401

    # Module 'reactor' has no 'run'/'stop' member, pylint: disable=E1101

    # Unused variable 'window', 'icon', pylint: disable=W0612
    icon, window = start(reactor.stop,
                         minimized=minimized, with_icon=with_icon,
                         auto_update_cb=autoupdate,
                         auto_update_interval=AUTOUPDATE_INTERVAL)
    # pylint: enable=W0612
    if icon:
        app.new_instance.connect(icon.restore_window)

    reactor.run()
    # pylint: enable=E1101
