# -*- coding: utf-8 -*-
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
"""System notification area icon."""

from PyQt4 import QtGui
from twisted.internet.defer import inlineCallbacks

from ubuntuone.platform.tools import SyncDaemonTool


class TrayIcon(QtGui.QSystemTrayIcon):

    """System notification icon."""

    def __init__(self, window=None):
        super(TrayIcon, self).__init__(None)
        self.setIcon(QtGui.QIcon(":/u1icon.png"))
        self.setVisible(True)
        self.window = window
        self.auto_update_lc = None
        self._message_cb = lambda: None
        self.messageClicked.connect(self.on_message_clicked)
        self.activated.connect(self.on_activated)
        self.context_menu = QtGui.QMenu()
        self.restore = QtGui.QAction("Restore", self,
            triggered=self.restore_window)
        self.quit = QtGui.QAction("Quit Ubuntu One", self,
            triggered=self.stop)
        self.context_menu.addAction(self.restore)
        self.context_menu.addSeparator()
        self.context_menu.addAction(self.quit)
        self.setContextMenu(self.context_menu)

    def on_activated(self, reason):
        """The user activated the icon."""
        if reason == self.Trigger:  # Left-click
            self.restore_window()

    def on_message_clicked(self):
        """Execute the callback that was set."""
        self._message_cb()

    def restore_window(self):
        """Show the main window."""
        if self.window is None:
            # pylint: disable=W0404
            from ubuntuone.controlpanel.gui.qt.gui import MainWindow
            # pylint: enable=W0404
            self.window = MainWindow(close_callback=self.delete_window)
        self.window.show()

    def delete_window(self):
        """Close and remove the main window."""
        if self.window is not None:
            self.window.close()
            self.window = None

    @inlineCallbacks
    def stop(self, *args, **kwargs):
        """Stop the application."""
        # Stop syncdaemon
        # pylint: disable=W0702
        # Really, if it fails we can't do anything about it.
        try:
            st = SyncDaemonTool()
            yield st.quit()
        except:
            # Maybe it was not running?
            pass
        # pylint: disable=W0404
        from twisted.internet import reactor
        # pylint: enable=W0404
        if self.auto_update_lc is not None and\
                self.auto_update_lc.running:
            self.auto_update_lc.stop()
        reactor.stop()

    def show_message(self, title, message, clicked_cb=lambda: None):
        """Show a message and set the new callback to execute."""
        self._message_cb = clicked_cb
        self.showMessage(title, message)
